/************************************************************************
 *
 *
 * Standalone test case for GEOSIntersects <hexwkbfile1> <hexwkbfile2>
 *
 *
 ***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "geos_c.h"

#define MAXHEXLEN 3000000

void
usage(char *me)
{
	fprintf(stderr, "Usage: %s <hexwkbfile1> <hexwkbfile2>\n", me);
	exit(1);
}

void
notice(const char *fmt, ...) {
	va_list ap;

	fprintf( stdout, "NOTICE: ");

	va_start (ap, fmt);
	vfprintf( stdout, fmt, ap);
	va_end(ap);
	fprintf( stdout, "\n" );
}

void
log_and_exit(const char *fmt, ...) {
	va_list ap;

	fprintf( stdout, "ERROR: ");

	va_start (ap, fmt);
	vfprintf( stdout, fmt, ap);
	va_end(ap);
	fprintf( stdout, "\n" );
	exit(1);
}

int
do_all(char *inputfile1, char *inputfile2)
{
	GEOSGeometry* g1;
	GEOSGeometry* g2;
	static char hex1[MAXHEXLEN];
	static char hex2[MAXHEXLEN];
	FILE *input1;
	FILE *input2;
	size_t size1;
	size_t size2;

	/* Read input files */
	input1 = fopen(inputfile1, "r");
	if ( ! input1 ) { perror("fopen file 1"); exit(1); }

	size1 = fread(hex1, 1, MAXHEXLEN-1, input1);
	fclose(input1);
	if ( ! size1 ) { perror("fread file 1"); exit(1); }
	if ( size1 == MAXHEXLEN-1 ) { perror("HEX input 1 too big!"); exit(1); }
	hex1[size1] = '\0'; /* ensure it is null terminated */

	input2 = fopen(inputfile2, "r");
	if ( ! input2 ) { perror("fopen file 2"); exit(2); }

	size2 = fread(hex2, 1, MAXHEXLEN-1, input2);
	fclose(input2);
	if ( ! size2 ) { perror("fread file 2"); exit(2); }
	if ( size2 == MAXHEXLEN-1 ) { perror("HEX input 2 too big!"); exit(2); }
	hex2[size2] = '\0'; /* ensure it is null terminated */

	/* Construct geoms from HEX WKB */
	g1 = GEOSGeomFromHEX_buf(hex1, size1);
	g2 = GEOSGeomFromHEX_buf(hex2, size2);

	/* Call intersects predicate */
	switch (GEOSIntersects(g1, g2))
	{
	case 0:
		printf("Intersects: false\n");
		break;
	case 1:
		printf("Intersects: true\n");
		break;
	default:
		log_and_exit("Exception running GEOSIntersects");
		break;
	}

	/* Clean up */
	GEOSGeom_destroy(g1);
	GEOSGeom_destroy(g2);

	return 0;
}

int
main(int argc, char **argv)
{
	initGEOS(notice, log_and_exit);
	printf("GEOS version %s\n", GEOSversion());
	if ( argc != 3 ) usage(argv[0]);

	do_all(argv[1], argv[2]);

	finishGEOS();
	return EXIT_SUCCESS;
}

