-- Create test schema and tables. Load test geometries in from text
-- files in HEX WKB. Note that this script must be run via psql
-- because it uses the psql \copy command.

create schema st_intersects_testcase;

create table st_intersects_testcase.linestring
(
    geom geometry(LineString, 4326)
);

create table st_intersects_testcase.mpoly
(
    geom geometry(MultiPolygon, 4326)
);

create table st_intersects_testcase.linestring_hex_wkb
(
    geom_hex_wkb text
);

create table st_intersects_testcase.mpoly_hex_wkb
(
    geom_hex_wkb text
);

-- The following LineString has 52228 points and has many
-- self-intersections.

\copy st_intersects_testcase.linestring_hex_wkb from 'linestring1_hex_wkb.txt';

-- The following MultiPolygon has 509 vertices and represents San
-- Francisco County in the United States.

\copy st_intersects_testcase.mpoly_hex_wkb from 'polygon1_hex_wkb.txt';

insert into st_intersects_testcase.mpoly
(select ST_GeomFromWKB(decode(geom_hex_wkb, 'hex'), 4326)
 from st_intersects_testcase.mpoly_hex_wkb);

insert into st_intersects_testcase.linestring
(select ST_GeomFromWKB(decode(geom_hex_wkb, 'hex'), 4326)
 from st_intersects_testcase.linestring_hex_wkb);

-- On our PostgreSQL server with 72GB of memory, the following query
-- runs for 20-60 minutes, consuming > 99% of one CPU until its
-- PostgreSQL backend process exhausts the available memory on the
-- machine. This causes the database to crash.
--
-- Debugging the PostgreSQL backend process under gdb shows that it is
-- inside the GEOSIntersects() call in the GEOS library during all
-- this time. GEOS appears to make millions of memory allocations on
-- the process private heap before the crash occurs.
--
-- WARNING: in our testing using PostgreSQL 9.3.5 on Ubuntu 12.04, it
-- is not possible to cancel this query with pg_cancel_backend(),
-- pg_terminate_backend(), or kill -TERM from Linux. The only reliable
-- way we found to cancel the query is to "kill -9" the PostgreSQL
-- backend process, which causes the postmaster to terminate all other
-- active server processes as well.

select count(*)
from
    st_intersects_testcase.linestring ls,
    st_intersects_testcase.mpoly mp
where
    ST_Intersects(ls.geom, mp.geom);
